# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/BioFusion',
                'doc_host': 'https://CalmScout.github.io',
                'git_url': 'https://github.com/CalmScout/BioFusion',
                'lib_path': 'BioFusion'},
  'syms': { 'BioFusion.cmmd': { 'BioFusion.cmmd.cmmd': ('cmmd.html#cmmd', 'BioFusion/cmmd.py'),
                                'BioFusion.cmmd.communities_detection': ('cmmd.html#communities_detection', 'BioFusion/cmmd.py'),
                                'BioFusion.cmmd.community_detection_ray': ('cmmd.html#community_detection_ray', 'BioFusion/cmmd.py'),
                                'BioFusion.cmmd.continual_multiplex_analysis': ( 'cmmd.html#continual_multiplex_analysis',
                                                                                 'BioFusion/cmmd.py')},
            'BioFusion.download_data': { 'BioFusion.download_data.create_dataframe_from_edges': ( 'download_data.html#create_dataframe_from_edges',
                                                                                                  'BioFusion/download_data.py')},
            'BioFusion.utils': { 'BioFusion.utils.create_and_save_random_graph': ( 'synthetic_data.html#create_and_save_random_graph',
                                                                                   'BioFusion/utils.py'),
                                 'BioFusion.utils.create_random_graph': ('synthetic_data.html#create_random_graph', 'BioFusion/utils.py'),
                                 'BioFusion.utils.generate_and_save_graphs': ( 'synthetic_data.html#generate_and_save_graphs',
                                                                               'BioFusion/utils.py'),
                                 'BioFusion.utils.obfuscate_nodes': ('synthetic_data.html#obfuscate_nodes', 'BioFusion/utils.py')}}}
